<?php
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
#############################################
#                                           #
#  ░░░░   ░░░░  ░░░░    ░░░░  ░░░░░░ ░░░░░░ #
# ░░  ░░ ░░  ░░  ░░      ░░   ░ ░░ ░ ░░     #
#  ░░    ░░  ░░  ░░      ░░     ░░   ░░     #
#   ░░   ░░  ░░  ░░      ░░     ░░   ░░░░░  #
#    ░░  ░░ ░░   ░░      ░░     ░░   ░░     #
# ░░  ░░ ░░  ░░  ░░ ░░   ░░     ░░   ░░     #
#  ░░░░   ░░░ ░ ░░░░░░  ░░░░   ░░░░  ░░░░░░ #
#                                           #
#############################################

/**
 ****************************************
 * Description
 * @name        : DB_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : Handles the SQlite DB access
 * @Version     : 1.1beta4
 ****************************************
 **/
namespace Helium\classes;
class DB{
	private $version="1.1beta3";
	
	private $db; # 
	private $tables; # array
	private $fileName; # string
	private $caseSensitive=false;

	public function __construct(&$db=null,string $fileName=null,array $tables=[]){
		if(!class_exists('SQLite3')){throw new Exception("SQLite 3 NOT supported.",1);}

		$this->fileName=$fileName;
		$this->tables=$tables;

		if (!$db && !$db=$this->connectToDb()){
			throw new Exception("Error connecting or creating db file",1);
		}

		$this->db=$db;

		foreach ($tables as $tableName=>$tableColumns) {
			$this->createTable($tableName);
		}
		
		return $this->db;
	}


	###########################
	# METHODS #################
	###########################
	public function connectToDb(string $fileName=null){
		if ($fileName){
			$this->fileName=$fileName;
		}
		if($this->fileName){
			$db=new \SQlite3($this->fileName);
			return $db;
		}
		return false;
	}

	public function createTable(string $tableName){
		if (!isset($this->tables[$tableName])){return false;}
		
		$command='CREATE TABLE IF NOT EXISTS '.$tableName.' (';
		$primarykey=' PRIMARY KEY';
		foreach ($this->tables[$tableName] as $column => $dataType) {
			$command.=' '.$column.' '.strtoupper($dataType).$primarykey.",";
			$primarykey='';
		}
		$command=trim($command,',').')';

		try {
		 	$this->db->exec($command);
		} catch (Exception $e) {
		 	throw new Exception("Error creating table $tableName: $e",1);
		} 

	}
	
	public function deleteTable(string $tableName){
		$command = "DROP TABLE $tableName";
		return @$this->db->exec($command);	
	}

	public function insertOne(string $tableName, array $dataArray){
		$data[0]=$dataArray;
		$this->insert($tableName,$data);
	}

	public function insert(string $tableName, array $dataArray){
		# dataArray FORMAT 
		# $dataArray=[
		#	0=>[col1=>value,col2=>'value' etc.]
		#	1=>[col1=>value,col2=>'value' etc.]
		# 	etc.
		$columnsName='';
		$columnsValue='';	
		$currentColumnsValue='';	
		$dataArray=$this->escape($dataArray);

		foreach ($dataArray as $index=>$data) {
			foreach ($data as $column => $value) {
				
				if (empty($index)){
					$columnsName.=$column.',';
				}

				if (is_string($value)){
					$currentColumnsValue.="'".$value."',";
				}else{
					$currentColumnsValue.=$value.",";
				}
				
			}
			$columnsValue.='('.trim($currentColumnsValue,',').'),';
			$currentColumnsValue='';
		}
		$columnsName=trim($columnsName,',');
		$columnsValue=trim($columnsValue,',');
		$command='INSERT or REPLACE INTO '.$tableName.'('.$columnsName.') VALUES '.$columnsValue.';';

		try {
		 	return $this->db->exec($command);
		} catch (Exception $e) {
		 	throw new Exception("Error executing insert data in database  [$command]",1);
		} 
	}

	public function update(string $tableName, string $condition, array $modifiedData){
		# update a column in a row
		# modifiedData FORMAT 
		# $modifiedData=['col1=value',"col2='value'" etc.]
		$condition=$this->escape($condition);
		$modifiedData=$this->escape($modifiedData);

		$command="UPDATE $tableName SET ";
		foreach ($modifiedData as $data) {			
			$command.=$data.",";
		}
		$command=trim($command,',');
		$command.=' WHERE ('.$condition.')';

		try {
		 	return $this->db->exec($command);
		} catch (Exception $e) {
		 	throw new Exception("Error executing update data in database [$command]",1);
		} 
	}
	private function conditions($conditions){
		$condition='';
		$conditions=$this->escape($conditions);
		if (is_string($conditions)){
			return $conditions;
		}
		foreach ($conditions as $cond) {	
			if (strpos($cond,'%')!==false){
				$cond=str_replace('=',' LIKE ',$cond);
			}
			$condition.=$cond." AND ";
		}
		$condition=str_replace(' AND ;','',$condition.';');
		return $condition;
	}
	public function search(string $tableName=null,$conditions,string $orderBy=''){
		# $condition = ['column="value"', 'column2="%partial%"',...]
		if (!$conditions||!$tableName){return false;}
		if (!empty($orderBy)){$orderBy=' ORDER BY '.$orderBy;}
		
		$condition=$this->conditions($conditions);
		if ($this->caseSensitive){$case=' COLLATE NOCASE';}else{$case='';}
		$command= "SELECT * FROM $tableName WHERE $condition $orderBy $case";

		$results=$this->db->query($command);

		if ($results){
			return $this->fetchAll($results);
		}
		return [];
	}

	public function get(string $tableName=null,string $wantedColumns='*',string $orderBy=''){
		# $wantedColumns ='column1,column2'	
		if (!$tableName){return false;}
		if (!empty($orderBy)){$orderBy=' ORDER BY '.$orderBy;}
		$command= "SELECT $wantedColumns FROM $tableName $orderBy";
		$results=$this->db->query($command);
		
		if ($results){
			return $this->fetchAll($results);
		}
		return [];
	}

	public function delete(string $tableName=null,$conditions){
		# $condition = ['column="value"', 'column2="%partial%"',...]	
		if (!$tableName){return false;}
		$condition=$this->conditions($conditions);

		$command= "DELETE FROM $tableName WHERE $condition";
		return $this->db->query($command);
	}

	public function exec(string $command){
		return $this->db->exec($command);
	}
	public function query(string $command){
		return $this->fetchAll($this->db->query($command));
	}

	public function fetchAll(object $results){
		$resultsArray=[];
		while ($row = $results->fetchArray(SQLITE3_ASSOC)) {
		    $resultsArray[]=$row;
		}
		return $resultsArray;
	}

	public function isTable(string $tableName){
		return isset($this->tables[$tableName]);
	}

	public function escape($data) {
        if(is_array($data))
            return array_map([$this,'escape'], $data);

        return \SQLite3::escapeString($data);
    }
	###########################
	# GETTERS #################
	###########################
	public function tables(){return $this->tables;}
	public function fileName(){return $this->fileName;}
	public function getVersion(){return $this->version;}
	public function getCaseSensitive(){return $this->caseSensitive;}
	
	###########################
	# SETTERS #################
	###########################
	public function setTables(array $tables){$this->tables=$tables;}
	public function setfileName(string $fileName){$this->fileName=$fileName;}
	public function setCaseSensitive(boolean $tf){$this->caseSensitive=$tf;}
	
}

?>