<?php
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
#############################################
#                                           #
#        ░░░░░  ░░░░░░  ░░░░  ░░░░░░        #
#        ░░  ░░ ░░     ░░  ░░ ░ ░░ ░        #
#        ░░  ░░ ░░      ░░      ░░          #
#        ░░░░░  ░░░░░    ░░     ░░          #
#        ░░  ░░ ░░        ░░    ░░          #
#        ░░  ░░ ░░     ░░  ░░   ░░          #
#        ░░  ░░ ░░░░░░  ░░░░   ░░░░         #
#                                           #
#############################################

/**
 ****************************************
 * Description
 * @name        : Helium_REST_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : handles & format the get/post/put/delete requests
 * @Version     : 1.0
 ****************************************
 **/
namespace Helium\classes;
class REST{
	private $version="1.0";
	
	private $status_codes 	=array(
								100 => 'Continue',  
								101 => 'Switching Protocols',  
								200 => 'OK',
								201 => 'Created',  
								202 => 'Accepted',  
								203 => 'Non-Authoritative Information',  
								204 => 'No Content',  
								205 => 'Reset Content',  
								206 => 'Partial Content',  
								300 => 'Multiple Choices',  
								301 => 'Moved Permanently',  
								302 => 'Found',  
								303 => 'See Other',  
								304 => 'Not Modified',  
								305 => 'Use Proxy',  
								306 => '(Unused)',  
								307 => 'Temporary Redirect',  
								400 => 'Bad Request',  
								401 => 'Unauthorized',  
								402 => 'Payment Required',  
								403 => 'Forbidden',  
								404 => 'Not Found',  
								405 => 'Method Not Allowed',  
								406 => 'Not Acceptable',  
								407 => 'Proxy Authentication Required',  
								408 => 'Request Timeout',  
								409 => 'Conflict',  
								410 => 'Gone',  
								411 => 'Length Required',  
								412 => 'Precondition Failed',  
								413 => 'Request Entity Too Large',  
								414 => 'Request-URI Too Long',  
								415 => 'Unsupported Media Type',  
								416 => 'Requested Range Not Satisfiable',  
								417 => 'Expectation Failed',  
								500 => 'Internal Server Error',  
								501 => 'Not Implemented',  
								502 => 'Bad Gateway',  
								503 => 'Service Unavailable',  
								504 => 'Gateway Timeout',  
								505 => 'HTTP Version Not Supported'); # array
	private $request		= []; # array
	private $method 		= ''; # string
	private $code 			= 200; # integer

	public function __construct(){
		$this->get_request();
	}

	###########################
	# METHODS #################
	###########################

	# get_referer
	###########################

	public function get_referer(){
		return $_SERVER['HTTP_REFERER'];
	}

	# get_status_message
	###########################

	public function get_status_message(){
		return ($this->status_codes[$this->code])?$this->status_codes[$this->code]:$this->status_codes[500];
	}

	# get_request_method
	###########################

	public function get_request_method(){
		return $_SERVER['REQUEST_METHOD'];
	}

	# get_data
	###########################

	public function get_request($formatted=false){
		$method=$this->get_request_method();
		switch($method){
			case 'POST':
				$this->request = ['method'=>$method,'data'=>$this->cleanInputs($_POST)];
				break;
			case 'GET': 
				$this->request = ['method'=>$method,'data'=>$this->cleanInputs($_GET)];
				break;
			case 'DELETE':
				$this->request = ['method'=>$method,'data'=>$this->cleanInputs($_GET)];
				break;
			case 'PUT':
				parse_str(file_get_contents("php://input"),$this->request);
				$this->request = ['method'=>$method,'data'=>$this->cleanInputs($this->request)];
				break;
			default:
				$this->response('',406);
				break;
		}
		if (!$formatted){ 
			return $this->request;
		}
		return $this->formatRequest($this->request);
	}

	private function formatRequest($request){
		$firstcommand=array_keys($request['data'])[0];
		$formattedRequest[strtolower($request['method'].$firstcommand)]=$request['data'][$firstcommand];
		return $formattedRequest;
	}

	# cleanInputs
	###########################
	## $data
	## Return : array
	public function cleanInputs($data){
		$clean_input = array();
		if(is_array($data)){
			foreach($data as $k => $v){
				$clean_input[$k] = $this->cleanInputs($v);
			}
		}else{
			if(get_magic_quotes_gpc()){
				$data = trim(stripslashes($data));
			}
			$data = strip_tags($data);
			$clean_input = trim($data);
		}
		return $clean_input;
	}

	# set_headers
	###########################

	private function set_headers(){
		header("HTTP/1.1 ".$this->code." ".$this->get_status_message());
		header("Content-Type:application/json");
	}

	public function response($data,$status){
		$this->code = ($status)?$status:200;
		$this->set_headers();
		echo json_encode($data);
		exit;
	}	

	###########################
	# GETTERS #################
	###########################
	public function status_codes(){return $this->status_codes;}
	public function request(){return $this->request;}
	public function method(){return $this->method;}
	public function code(){return $this->code;}
	public function content_type(){return $this->content_type;}
	public function getVersion(){return $this->version;}
}

?>