<?php
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
######################################
#                                    #
#  ░░░░   ░░░░   ░░░░  ░░  ░░ ░░░░░░ #
# ░░  ░░ ░░  ░░ ░░  ░░ ░░  ░░ ░░     #
# ░░     ░░  ░░ ░░     ░░  ░░ ░░     #
# ░░     ░░░░░░ ░░     ░░░░░░ ░░░░░  #
# ░░     ░░  ░░ ░░     ░░  ░░ ░░     #
# ░░  ░░ ░░  ░░ ░░  ░░ ░░  ░░ ░░     #
#  ░░░░  ░░  ░░  ░░░░  ░░  ░░ ░░░░░░ #
#                                    #
######################################

/**
 ****************************************
 * Description
 * @name        : Helium_locale_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : cache functions
 * @Version     : 1.0
 * @use 		:
 ****************************************
 **/

namespace Helium\classes;

class cache{
	private $version="1.0";
	private $filename;
	private $cache_enabled=true;
	private $data;
	public  $message;
	private $cache_path;
	
	public function __construct($path='temp/'){
		$this->cache_path=$path;
		if (
			  $this->cache_path
			&&!is_dir($this->cache_path)
		){
			mkdir ($this->cache_path);
		}
	}

	###########################
	# METHODS #################
	###########################

	# start
	###########################
	# return true if cache file exists or false if not
	public function start($filename=false){
		if (
			!$this->cache_enabled
		){
			$this->filename=false;
			return true;
		}

		# File exists=>load it and return false (don't regenerate page)
		if (is_file($this->cache_path.$filename)){
			$this->read($filename);
			$this->message='Cache version';
			return false;
		}

		ob_start();
		$this->filename=$filename;
		return true;
	}

	# end
	###########################
	public function end(){
		if (!$this->data){
			$this->data=ob_get_clean();
			$this->write($this->filename,$this->data);
			$this->message='';
		}
		
		echo $this->data;
	}

	# delete
	###########################
	public function delete(string $filename=null){
		if (!$filename){return;}
		$filelist=glob($this->cache_path.$filename.'*');
		foreach($filelist as $file){
			if (is_file($file)){unlink($file);}
		}
	}

	# read
	###########################
	private function read(string $file=null){
	  if(!$file){return false;}	  
	  if (file_exists($this->cache_path.$file)){
	    $this->data=file_get_contents($this->cache_path.$file);
	    return $this->data; 
	  }else{return false;}
	}

	# write
	###########################
	private function write(string $file=null,$data=null){
	  if(!$file||!$data){return false;}	  
	  file_put_contents($this->cache_path.$file,$data);	  
	}

	# clear
	###########################
	public function clear(){
	  $fs=glob($this->cache_path.'*');
	  foreach ($fs as $file){
	    unlink ($file);
	  }
	}
	
	# disableCache
	###########################
	public function disableCache(){
	  $this->cache_enabled=false;
	}
	# enableCache
	###########################
	public function enableCache(){
	  $this->cache_enabled=true;
	}
	public function getVersion(){return $this->version;}
}
