<?php 
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
#############################################
#                                           #
#  ░░░░   ░░░░  ░   ░░ ░░░░░░  ░░░░   ░░░░  #
# ░░  ░░ ░░  ░░ ░░  ░░ ░░       ░░   ░░  ░░ #
# ░░     ░░  ░░ ░░░ ░░ ░░       ░░   ░░     #
# ░░     ░░  ░░ ░░░░░░ ░░░░░    ░░   ░░ ░░░ #
# ░░     ░░  ░░ ░░ ░░░ ░░       ░░   ░░  ░░ #
# ░░  ░░ ░░  ░░ ░░  ░░ ░░       ░░   ░░  ░░ #
#  ░░░░   ░░░░  ░░   ░ ░░      ░░░░   ░░░░  #
#                                           #
#############################################
/**
 ****************************************
 * Description
 * @name        : Helium_config_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : All config access (create, edit, generate form, save/load)
 * @Version     : 1.2.2 
 * @use: $cfg->get('varname') , $cfg->set('varname', $value);
 ****************************************
 **/

namespace Helium\classes;
class config{
	private $version="1.2.1";
	
	private $context; # reference
	private $tpl;
	private $secure;
	private $locale;

	private $config; # array	
	private $config_format=array(); #array
	
	private $elements=[
		'select'=>'<li><label for="#key"><span>#varname</span>	<p>#description</p></label><span><select name="#key" #required>#options</select></span></li>',
		'checkbox'=>'<li><label for="#key"><span>#varname</span><p>#description</p></label><input name="#key" type="hidden" id="#key" value="0"><input name="#key" type="checkbox" id="#key" value="1"></li>',
		'input'=>'<li><label for="#key"><span>#varname</span><p>#description</p></label><input name="#key" type="#type" id="#key" #required #value></li>',
		'submit'=>'<li><span></span><input type="submit" value="&#9989;"/></li>',
	];
	
	public function __construct($cfg_path='.data/config',$config_format=[]){
		$this->setConfigFormat($config_format);
		

		$this->load();
		$this->check();
	}

	###########################
	# METHODS #################
	###########################


	# check
	###########################
	# sanitize & check configuration integrity
	private function check(){
		$save=false;
		# complete config data if needed
		foreach ($this->config_format as $key => $data) {
			if (!isset($this->config[$key])){
				$this->config[$key]=$data['default_value'];
				$save=true;
			}
			$this->config[$key]=\tools::sanitizeText($this->config[$key]);
		}

		if ($save){			
			$this->save();
		}
	}

	# sort
	###########################
	# sort configs keys
	public function sort(){
		ksort($this->config);
		return $this->config;
	}	

	# save
	###########################
	# save configuration
	## Return : boolean
	public function save(){
		return \tools::save(CONFIG.'config.php',$this->config);
	}

	# load
	###########################
	# load configuration array
	private function load(){
		$c=\tools::load(CONFIG.'config.php');		
		$this->config=$c;
		return $c;
	}
	
	# renderForm
	# render the configuration form
	public function renderForm(){
		if (empty($this->tpl)){
			$this->tpl=\tools::getClassInstance($this->context,'tpl');
		}
		
		$inputtoken=!empty($this->secure)?$this->secure->inputToken():'';

		$form='<form action="#" method="POST"><input type="hidden" name="config"/>'.$inputtoken;
        foreach ($this->config as $key => $value) {
        	$attributes=$this->getAttributes($key);

        	$type=!empty($this->config_format[$key]['type'])?$this->config_format[$key]['type']:'';
        	$default_value=!empty($this->config_format[$key]['default_value'])?$this->config_format[$key]['default_value']:'';
        	$current_value=!empty($this->config[$key])?$this->config[$key]:$default_value;
        	$description=!empty($this->config_format[$key]['description'])?$this->config_format[$key]['description']:'';
        	$other_values=!empty($this->config_format[$key]['other_values'])?$this->config_format[$key]['other_values']:[];
        	$other_values[]=$default_value;
        	

        	if ($type=='select'){
        		$options='';
        		foreach ($other_values as $value) {
    				$active=($value==$current_value)?' selected ':'';
    				$options.='<option value="'.$value.'"'.$active.'>'.$value.'</option>';
    			}
        		$form.=$this->renderElement('select',['#attributes'=>$attributes,'#varname'=>str_replace('_',' ',$key),'#key'=>$key,'#description'=>$description,'#options'=>$options,'#value'=>$current_value]);
        	}elseif($type=='checkbox'){
        		$active=($value==1)?' checked ':'';
        		$form.=$this->renderElement('checkbox',['#attributes'=>$attributes,'#varname'=>str_replace('_',' ',$key),'#key'=>$key,'#description'=>$description,'#value'=>$active]);
        	}else{
        		$form.=$this->renderElement('input',['#attributes'=>$attributes,'#varname'=>str_replace('_',' ',$key),'#key'=>$key,'#description'=>$description,'#type'=>$type,'#value'=>$current_value]);
        	}
        }
        
        $form.=$this->renderElement('submit');'</form>';
        return $form;
    }

    # render the html from templates: uses tpl class if in context or default templates if not
	private function renderElement($element,$data=[]){
		if ($this->tpl){        			
			return $this->tpl->render($element,$data);
		}else{
			return str_replace(array_keys($data),array_values($data),$this->elements[$element]);
		}
	}
    # remove all data from config_format[key] and keep only attributes to add
	private function getAttributes($key){
		$attributes=$this->config_format[$key];
		unset($attributes['other_values']);
		unset($attributes['type']);
		unset($attributes['default_value']);
		unset($attributes['description']);
		$attr='';
		foreach ($attributes as $key => $value) {
			$attr.=' '.$key.'="'.$value.'"';
		}
		return $attr;
	}

	# handlePost
	# saves the form changes
	public function handlePost(){
        # detect if ajax or normal request for the results
		if (isset($_POST['config'])){
			$posted_data=$_POST;
			$posted_data['app_name']=strip_tags($this->context['RAW_POST']['app_name']);
			$posted_data['app_description']=strip_tags($this->context['RAW_POST']['app_description']);
			$posted_data['home_msg']=strip_tags($this->context['RAW_POST']['home_msg']);
			
			unset($posted_data['config']);
			if ($this->secure){$this->secure->lock();}
			# save the data
			unset($posted_data['token']);
			$this->config=$posted_data;
			$this->save();
			$_POST=[];
			if (\tools::isAjax()){
				header("HTTP/1.1 200 OK");
				exit;
			}
			
			
		}


    }

	###########################
	# GETTERS #################
	###########################
	public function getConfig(){return $this->config;}
	public function getConfigFormat(){return $this->config_format;}

	# get
	###########################
	# return config[index] value
	public function get(string $index=null){
		if ($index&&isset($this->config[$index])){
			return $this->config[$index];
		}
		return false;
	}


	###########################
	# SETTERS #################
	###########################
	public function setContext(&$context=null){					
		$this->context=&$context;
		$this->tpl=\tools::getClassInstance($this->context,'tpl');
		$this->secure=\tools::getClassInstance($context,'secure');
		$this->locale=\tools::getClassInstance($context,'locale');
	}	
	public function setConfig(array $value){
		if (empty($value)){
			$this->load();
		}else{
			$this->config=$value;
			$this->save();
		}
	}

	public function setConfigFormat(array $value){
		$this->config_format=$value;
	}

	# set
	###########################
	# change config[index] value
	public function set($index=null,$val=null){
		if ($index&&$val){
			$this->config[$index]=$val;
		}		
	}
	public function getVersion(){return $this->version;}


}
