<?php
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
########################
#                      #
# ▒▒▒▒▒   ▒▒▒▒   ▒▒▒▒  #
# ▒▒  ▒▒ ▒▒  ▒▒ ▒▒  ▒▒ #
# ▒▒  ▒▒  ▒▒     ▒▒    #
# ▒▒▒▒▒    ▒▒     ▒▒   #
# ▒▒  ▒▒    ▒▒     ▒▒  #
# ▒▒  ▒▒ ▒▒  ▒▒ ▒▒  ▒▒ #
# ▒▒  ▒▒  ▒▒▒▒   ▒▒▒▒  #
#                      #
########################


/**
 ****************************************
 * Description
 * @name        : Helium_RSS_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : All RSS feed features (generate/read)
 * @Version     : 1.1 [added header rss to doFeed()]
 ****************************************
 **/
namespace Helium\classes;
class rss{
	private $version="1.1";
	private $xml_file_duration=60; # seconds
	private $xml_file_name='feed.xml';
	private $title;
	private $type='rss';
	private $author;
	private $url;
	private $cache=false;
	private $description='';
	private $items=[];
	
	public function __construct($data=[]){
		if ($data){
			foreach ($data as $key => $value) {
				if (property_exists($this, $key)){
					$this->$key=$value;
				}
			}
		}

		if (defined('APP_NAME')&&empty($this->title)){
			$this->title=APP_NAME.' RSS feed';
			$this->description=APP_NAME.' RSS feed';
		}

		if (defined('APP_AUTHOR')&&empty($this->author)){
			$this->author=APP_AUTHOR;
		}


	}

	###########################
	# METHODS #################
	###########################
	public function addItem($data=[]){
		if (empty($data['title'])){			throw new Exception("Rss error adding item: no «title» !", 1);}
		if (empty($data['link'])){			throw new Exception("Rss error adding item: no «link» !", 1);}
		if (empty($data['description'])){	throw new Exception("Rss error adding item: no «description» !", 1);}
		if (empty($data['date'])){			
			$data['date']=$this->makeRSSdate();
		}else{
			$data['date']=$this->makeRSSdate($data['date']);
		}
		$this->items[]=$data;
	}

	public function makeRSSdate($date=NULL){// format DD-MM-YY HH:MM => rfc2822 (rss compatible)

        if ($date){
            date_default_timezone_set('Europe/Paris');
            $time = strtotime($date);
            return date("r", $time);
        }else{
        	return date("r");
        }
    }

    public function doFeed(){
		if (empty($this->items)){
			throw new Exception("Rss error creating xml file: there's no items !", 1);
		}
		$feed='';

		$r="\n";$t="\t";
		$tpl=['rss'=>[],'atom'=>[]];
		$tpl['rss']['header']=
			'<?xml version="1.0" encoding="utf-8" ?>'.$r
			.'<rss version="2.0"  xmlns:content="http://purl.org/rss/1.0/modules/content/">'.$r.$t
			.'<channel>'.$r;
		$tpl['atom']['header']='<feed xmlns="http://www.w3.org/2005/Atom">'.$r;
		$tpl['rss']['footer']=$t.'</channel></rss>'.$r;
		$tpl['atom']['footer']='</feed>'.$r;
		$tpl['rss']['content-type']='Content-Type: application/rss+xml; charset=utf-8';
		$tpl['atom']['content-type']='Content-Type: application/atom+xml; charset=utf-8';

		header($tpl[$this->type]['content-type']);
    	if (
    		is_file($this->xml_file_name)
    		&& filemtime($this->xml_file_name)+$this->xml_file_duration<time()
    		&&$this->cache
    	){
    		exit(file_get_contents($this->xml_file_name));
    	}

		$feed=$tpl[$this->type]['header'];
		//create the feed's info content
		
		if ($this->type=='atom'){ // ATOM
			$feed.=$t.$t.'<link href="'.$this->url.'" rel="self" type="application/atom+xml"/>'.$r;
			$feed.=$t.$t.'<author><name>'.$this->author.'</name></author>'.$r;
		}
		$feed.=$t.$t.'<title>'.$this->title.'</title>'.$r;
		$feed.=$t.$t.'<link>'.$this->url.'</link>'.$r;
		$feed.=$t.$t.'<description>'.$this->description.'</description>'.$r;

		

		//then the items content
		foreach ($this->items as $item){
			if ($this->type=='atom'){ 
				$feed.=$t.$t.$t.'<entry>'.$r;}else{$feed.=$t.$t.$t.'<item>'.$r;
			}
			foreach($item as $key=>$value){
				if ($this->type=='atom'){ // ATOM
					if ($key=='link'){
						$feed.=$t.$t.$t.$t.'<link href="'.$value.'" rel="alternate" type="text/html"/>'.$r;		
					}
					elseif ($key=='content'){
						$feed.=$t.$t.$t.$t.'<content type="text">'.htmlspecialchars($value).'</content>'.$r;
					}
					elseif (!empty($value)){
						$feed.=$t.$t.$t.$t.'<'.$key.'>'.no_nl($value).'</'.$key.'>'.$r;
					}
				}else{ // RSS
					if ($key=='pubDate'||$key=='title'||$key=='link'){
						$feed.=$t.$t.$t.$t.'<'.$key.'>'.htmlspecialchars($value).'</'.$key.'>'.$r;
					}elseif ($key=='date'){

					}
					elseif($key=='guid'){ 
						$feed.=$t.$t.$t.$t.'<guid isPermaLink="false"><![CDATA['.$value.']]></guid>'.$r;
					}elseif(!empty($value)){
						$feed.=$t.$t.$t.$t.'<'.$key.'><![CDATA['.\tools::no_nl($value).']]></'.$key.'>'.$r;
					}
				}
			}
			if ($this->type=='atom'){ 
				$feed.=$t.$t.$t.'</entry>'.$r;
			}else{
				$feed.=$t.$t.$t.'</item>'.$r;
			}
		}


		$feed.=$tpl[$this->type]['footer'];
		file_put_contents($this->xml_file_name, $feed);
		header('Content-Type: text/xml');
		exit($feed);
	}

    # feed2Array
    ##############################
    # get a rss feed and return an array
    public function feed2Array($url=null,$return_items_only=false){
    	$url=empty($url)?$this->url:$url;
    	if (empty($url)){ return false;}

		$feed_content=\tools::curlContent($url);

		$feed_array=['infos'=>[],'items'=>[]];

		if(preg_match('~<rss(.*)</rss>~si', $feed_content)){
			$type='RSS';
		}
		elseif(preg_match('~<feed(.*)</feed>~si', $feed_content)){
			$type='ATOM';
		}
		else return false;//if the feed isn't rss or atom

		$feed=$this->type=$type;
		//ini_set('display_errors', '0'); 
		$feed_content=trim($feed_content," \t\n\r\0\x0B");
		try{
			$feed_obj = new SimpleXMLElement($feed_content, LIBXML_NOCDATA);
		}catch (Exception $e){return false;}	

		//ini_set('display_errors', '1');
		$feed_array['infos']['version']=$feed_obj->attributes()->version;
		if (!empty($feed_obj->attributes()->version)){	$feed_array['infos']['version']=(string)$feed_obj->attributes()->version;}
		if (!empty($feed_obj->channel->title)){			$feed_array['infos']['title']=(string)$feed_obj->channel->title;}
		if (!empty($feed_obj->channel->subtitle)){		$feed_array['infos']['subtitle']=(string)$feed_obj->channel->subtitle;}
		if (!empty($feed_obj->channel->link)){			$feed_array['infos']['link']=(string)$feed_obj->channel->link;}
		if (!empty($feed_obj->channel->description)){	$feed_array['infos']['description']=(string)$feed_obj->channel->description;}
		if (!empty($feed_obj->channel->language)){		$feed_array['infos']['language']=(string)$feed_obj->channel->language;}
		if (!empty($feed_obj->channel->copyright)){		$feed_array['infos']['copyright']=(string)$feed_obj->channel->copyright;}

		if (!empty($feed_obj->title)){					$feed_array['infos']['title']=(string)$feed_obj->title;}
		if (!empty($feed_obj->subtitle)){				$feed_array['infos']['subtitle']=(string)$feed_obj->subtitle;}
		if (!empty($feed_obj->link)){					$feed_array['infos']['link']=(string)$feed_obj->link;}
		if (!empty($feed_obj->description)){			$feed_array['infos']['description']=(string)$feed_obj->description;}
		if (!empty($feed_obj->language)){				$feed_array['infos']['language']=(string)$feed_obj->language;}
		if (!empty($feed_obj->copyright)){				$feed_array['infos']['copyright']=(string)$feed_obj->copyright;}
		
		if (!empty($feed_obj->channel->item)){
			foreach ($feed_obj->channel->item as $key => $value) {
				$items[]=$value;
			}
		}
		if (!empty($feed_obj->entry)){			$items=$feed_obj->entry;}
		if (empty($items)){return false;}
		foreach ($items as $key=>$item){	
			$medias='';	
			$c=count($feed_array['items']);
			if (count($item->enclosure)){
				// medias attached				
				foreach ($item->enclosure as $key => $media_object) {
					$media=$media_object->attributes();
					if (stripos($media['type'],'image')!==false){
						$medias.='<img src="'.$media['url'].'" alt="'.$media['url'].'"/>';
					}
					elseif (stripos($media['type'],'video')!==false){
						$medias.='<video controls src="'.$media['url'].'" alt="'.$media['url'].'"></video>';
					}
					elseif (stripos($media['type'],'audio')!==false){
						$medias.='<audio controls src="'.$media['url'].'" alt="'.$media['url'].'"></audio>';
					}
					else{
						$medias.='<a href="'.$media['url'].'">'.basename($media['url']).'</a>';
					}
				}
				$medias="<section>$medias</section>";
				
			}
			if(!empty($item->title)){		 	$feed_array['items'][$c]['title'] 	  	= (string)$item->title;}
			if(!empty($item->logo)){		 	$feed_array['items'][$c]['titleImage']  = (string)$item->logo;}
			if(!empty($item->icon)){		 	$feed_array['items'][$c]['icon'] 		= (string)$item->icon;}
			if(!empty($item->link['href'])){ 	$feed_array['items'][$c]['link']		= (string)$item->link['href'];}
			if(!empty($item->language)){		$feed_array['items'][$c]['language']	= (string)$item->language;}
			if(!empty($item->author->name)){ 	$feed_array['items'][$c]['author']	  	= (string)$item->author->name;}
			if(!empty($item->author->email)){	$feed_array['items'][$c]['email'] 	  	= (string)$item->author->email;}
			if(!empty($item->updated)){			$feed_array['items'][$c]['last'] 		= (string)$item->updated;}
			if(!empty($item->rights)){			$feed_array['items'][$c]['copyright']	= (string)$item->rights;}
			if(!empty($item->generator)){		$feed_array['items'][$c]['generator']	= (string)$item->generator;}
			if(!empty($item->guid)){			$feed_array['items'][$c]['guid']	 	= (string)$item->guid;}
			if(!empty($item->pubDate)){			$feed_array['items'][$c]['pubDate']	  	= (string)$item->pubDate;$feed_array['items'][$c]['date'] =(string)$item->pubDate;}
			if(!empty($item->published)){		$feed_array['items'][$c]['date']		= (string)$item->published;}
			if(!empty($item->update)){			$feed_array['items'][$c]['update'] 	  	= (string)$item->update;}
			if(!empty($item->link)){			$feed_array['items'][$c]['link'] 	  	= (string)$item->link;}
			if(!empty($item->summary)){			$feed_array['items'][$c]['description'] = (string)$item->summary.$medias;}
			if(!empty($item->subtitle)){ 		$feed_array['items'][$c]['description'] = (string)$item->subtitle.$medias;}
			if(!empty($item->description)){		$feed_array['items'][$c]['description'] = (string)$item->description.$medias;}
			if(!empty($item->content)){			$feed_array['items'][$c]['description'] = (string)$item->content.$medias;}
		}	

		$this->items 		=$feed_array['items'];
		$this->title 		=$feed_array['infos']['title'];
		$this->description 	=$feed_array['infos']['description'];
		$this->url  		=$url;
		
		if ($return_items_only){
			return $this->items;
		}
		return $feed_array;
	}


	###########################
	# Getters #################
	###########################
    public function getVersion(){						return $this->version;}
    public function getTitle(){							return $this->title;}
    public function getUrl(){							return $this->url;}
    public function getType(){							return $this->type;}
    public function getAuthor(){						return $this->author;}
    public function getDescription(){					return $this->description;}
    public function getXmlFileName(){					return $this->xml_file_name;}
    public function getCacheStatus(){					return $this->cache;}
    public function getItems(){							return $this->items;}


	###########################
	# Setters #################
	###########################
    public function setTitle(string $value){			$this->title=$value;}
    public function setUrl(string $value){				$this->url=$value;}
    public function setType(string $value){				$this->type=strtolower($value);}
    public function setAuthor(string $value){			$this->author=$value;}
    public function setDescription(string $value){		$this->description=$value;}
    public function setXmlFileName(string $value){		$this->xml_file_name=$value;}
    public function setCacheStatus(boolean $value){		$this->cache=$value;}
    public function setItems(array $value){			$this->items=$value;}

}