<?php

##############################################
#                                            #
# ░░░░░░ ░░  ░░ ░░  ░░ ░     ░ ░░░░░   ░░░░  #
# ░ ░░ ░ ░░  ░░ ░░  ░░ ░░   ░░ ░░  ░░ ░░  ░░ #
#   ░░   ░░  ░░ ░░  ░░ ░░░ ░░░ ░░  ░░  ░░    #
#   ░░   ░░░░░░ ░░  ░░ ░░░░░░░ ░░░░░    ░░   #
#   ░░   ░░  ░░ ░░  ░░ ░░ ░ ░░ ░░  ░░    ░░  #
#   ░░   ░░  ░░ ░░  ░░ ░░   ░░ ░░  ░░ ░░  ░░ #
#  ░░░░  ░░  ░░  ░░░░  ░░   ░░ ░░░░░   ░░░░  #
#                                            #
##############################################

/**
 ****************************************
 * Description
 * @name        : Helium_thumbs_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : Generation, return, handle thumbnails
 * @Version     : 1.2
 ****************************************
 **/
namespace Helium\classes;
class thumbs{
	private $thumb_size=128; # integer
	private $thumb_prefix='_THUMB_'; # string
	private $thumb_path=DATA.'thumbs/'; # string
	private $gallery_thumb_size=250; # integer
	private $gallery_thumb_prefix='_GALTHUMB_'; # string
	private $crop_image=true; # boolean
	private $output_format; # string
	private $BZN;
	
	public function __construct(){
		define('EXIF_ORIENTATION_TOP',1);
		define('EXIF_ORIENTATION_BOTTOM',3);
		define('EXIF_ORIENTATION_RIGHT',6);
		define('EXIF_ORIENTATION_LEFT',8);
		if (!is_dir($this->thumb_path)){
			mkdir($this->thumb_path);
		}
	}

	###########################
	# METHODS #################
	###########################

	private function getRotationAngle($exif_orientation){
		if (EXIF_ORIENTATION_BOTTOM == $exif_orientation){return 180;}
		if (EXIF_ORIENTATION_RIGHT == $exif_orientation){return 270;}
		if (EXIF_ORIENTATION_LEFT == $exif_orientation){return 90;}
		return 0;
	}

	# fileName
	###########################
	# return thumb filename
	## $path as string
	## $gallery as boolean
	## Return : string
	public function fileName($path,$gallery=false){
		$size=$this->size($gallery);
		if ($gallery){
			$prefix=$this->gallery_thumb_prefix;
		}else{
			$prefix=$this->thumb_prefix;
		}
		$pattern='#\.(jpe?g|png|gif)#i';
		$replace=$prefix.$size.'x'.$size.'.$1';		
		return preg_replace($pattern,$replace,$path);
	}

	# size
	###########################
	# return thumb size
	## $path as string
	## $gallery as boolean
	## Return : string
	public function size($gallery=false){
		if ($gallery){
			return $this->gallery_thumb_size;
		}else{
			return $this->thumb_size;
		}
	}


	# path
	###########################
	## $path as string
	## Return : string
	public function path($img=null,$gallery=false,$just_path=false){
		if (!$img){
			return false;
		}
		$size 		=$this->size($gallery);
		$thumb_name =THUMBS_PATH.$this->fileName($img,$gallery);

		# image doesn't exists
		if (!is_file($img)){
			# delete useless thumb
			if (is_file($thumb_name)){
				unlink($thumb_name);
			}
			# return error
			if (!$just_path){				
				return 'THUMBNAIL ERROR: '.$img.' doesn\'t exists';
			}else{
				return false;
			}
		}

		if (is_file($thumb_name)){
			return $thumb_name;
		}elseif(!$just_path){
			return $this->do($img,$thumb_name,$size,$gallery);
		}
		
	}

	# generate
	###########################
	## $path as string
	## $gallery as boolean
	## Return : string
	public function do($img=null,$thumb_name=null,$size=null,$gallery=false){
		
		if(stripos($img,'.svg')==strlen($img)-4){
			return $img;
		}

		// initialization
		if (!getimagesize($img)||!$img){
			return false;
		}
		if (!$thumb_name){
			$thumb_name=$this->fileName($img,$gallery);
		}
		if (strpos($thumb_name,$this->thumb_path)===false){
			$thumb_name=$this->thumb_path.'-'.hash('md2',$img).'-'.basename($thumb_name);
		}
		if (!$size){$size=$this->thumb_size;}
		
		$Xcrop=0;$Ycrop=0;
		
		# Crop
		$img_size = getimagesize($img);
		$src_width=$img_size[0];
		$src_height=$img_size[1];
		if (!$this->crop_image){
			if ($src_width<=$src_height){
				// portrait
				$ratio=$src_height/$src_width;
				$sizeH=$size;
				$sizeW=$size/$ratio;
			}else if ($src_width>$src_height){
				// landscape
				$ratio=$src_width/$src_height;
				$sizeW=$size;
				$sizeH=$size/$ratio;
			}
		}else{
			
			if ($src_width<=$src_height){
				// portrait
				$Ycrop=round(($src_height-$src_width)/2);
				$src_height=$src_width;
			}else if ($src_width>$src_height){
				// paysage
				$Xcrop=round(($src_width-$src_height)/2);
				$src_width=$src_height;
			}
			
			$sizeW=$sizeH=$size;
		}

		if (empty($type)){
			$type=\tools::getMime($img);
		}
		$type=explode('/',$type);
		$extension=str_ireplace('jpg','jpeg',$type[1]);
		
		$function='imagecreatefrom'.$extension;
		if (!$src  = $function($img)){return false;}
		$thumb = imagecreatetruecolor($sizeW,$sizeH);


		# output format forced ?
		if (!empty($this->output_format)){
			$extension=$this->output_format;
		}

		# transparency
		# (see Seebz'function: http://code.seebz.net/p/imagethumb/)
		if( $extension=='png' ){
			imagealphablending($thumb,false);
			imagesavealpha($thumb,true);
		}
		if( $extension=='gif'  && @imagecolortransparent($src)>=0 ){
			$transparent_index = @imagecolortransparent($src);
			$palletsize = imagecolorstotal($src);
		    if ($transparent_index >= 0 && $transparent_index < $palletsize) {
		      $transparent_index = imagecolorsforindex($src, $transparent_index);
		  	}
			$transparent_index = imagecolorallocate($thumb, $transparent_color['red'], $transparent_color['green'], $transparent_color['blue']);
			imagefill($thumb, 0, 0, $transparent_index);
			imagecolortransparent($thumb, $transparent_index);
		}
		
		imagecopyresampled($thumb,$src,0,0,$Xcrop,$Ycrop,$sizeW,$sizeH,$src_width,$src_height);

		# rotation
		if( $extension=='jpg'){
			@$exif = exif_read_data($img);
			if ($exif && array_key_exists('Orientation', $exif)) {
				$orientation = $exif['Orientation'];
				$angle = $this->getRotationAngle($orientation);
				$thumb = imagerotate($thumb, $angle, 0);
			}
		}


		if( $extension=='png' ){imagepng($thumb, $thumb_name);}
		else{imagejpeg($thumb, $thumb_name);}
		imagedestroy($thumb);
		return $thumb_name;
	}

	
	###########################
	# SETTERS #################
	###########################
	public function setOutputFormat($value){
		if (stripos('jpg jpeg png gif', $value)===false){
			return;
		}
		$this->output_format=$value;
	}
	public function setCropImage($value=true){
		if (!is_bool($value)){$value=true;}
		$this->crop_image=$value;
	}
	public function setThumbSize($value){
		if (!is_integer($value)){$value=intval($value);}
		$this->thumb_size=$value;
	}
	public function setThumbPrefix($value){
		if (!is_string($value)){
			trigger_error('Value must be string',E_USER_WARNING);
			return;
		}
		$this->thumb_prefix=$value;
	}
	public function setGalleryThumbSize($value){
		if (!is_integer($value)){
			trigger_error('Value must be integer',E_USER_WARNING);
			return;
		}
		$this->gallery_thumb_size=$value;
	}
	public function setGalleryThumbPrefix($value){
		if (!is_string($value)){
			trigger_error('Value must be string',E_USER_WARNING);
			return;
		}
		$this->gallery_thumb_prefix=$value;
	}

	###########################
	# GETTERS #################
	###########################
	public function getOutputFormat(){return $this->output_format;}
	public function getThumbSize(){return $this->thumb_size;}
	public function getThumbPrefix(){return $this->thumb_prefix;}
	public function getGalleryThumbSize(){return $this->gallery_thumb_size;}
	public function getGalleryThumbPrefix(){return $this->gallery_thumb_prefix;}



}

?>