<?php
##############################################
#                                            #
# ██  ██ ██████ ████    ████  ██  ██ █     █ #
# ██  ██ ██      ██      ██   ██  ██ ██   ██ #
# ██  ██ ██      ██      ██   ██  ██ ███ ███ #
# ██████ █████   ██      ██   ██  ██ ███████ #
# ██  ██ ██      ██      ██   ██  ██ ██ █ ██ #
# ██  ██ ██      ██ ██   ██   ██  ██ ██   ██ #
# ██  ██ ██████ ██████  ████   ████  ██   ██ #
#                                            #
##############################################
######################################
#                                    #
# ▒▒  ▒▒  ▒▒▒▒  ▒▒▒▒▒▒ ▒▒▒▒▒   ▒▒▒▒  #
# ▒▒  ▒▒ ▒▒  ▒▒ ▒▒     ▒▒  ▒▒ ▒▒  ▒▒ #
# ▒▒  ▒▒  ▒▒    ▒▒     ▒▒  ▒▒  ▒▒    #
# ▒▒  ▒▒   ▒▒   ▒▒▒▒▒  ▒▒▒▒▒    ▒▒   #
# ▒▒  ▒▒    ▒▒  ▒▒     ▒▒  ▒▒    ▒▒  #
# ▒▒  ▒▒ ▒▒  ▒▒ ▒▒     ▒▒  ▒▒ ▒▒  ▒▒ #
#  ▒▒▒▒   ▒▒▒▒  ▒▒▒▒▒▒ ▒▒  ▒▒  ▒▒▒▒  #
#                                    #
######################################


/**
 ****************************************
 * Description
 * @name        : Helium_users_class.php
 * @Package     : Helium
 * @Author      : Bronco
 * @Description : All users access (create, edit, save/load) [needed for secure class]
 * @Version     : 1.1
 ****************************************
 **/
# EN COURS: refaire toutes les methodes puis les utiliser dans secure
namespace Helium\classes;
class users{
	private $users_folder_path;
	private $users;
	private $context;
	private $secure;
	
	public function __construct($users_folder_path='./data/secure/'){
		$this->users_folder_path=$users_folder_path;
		$this->load();
	}

	###########################
	# METHODS #################
	###########################


	# deleteUser
	###########################
	public function deleteUser($user=null,$save=true){
		if (
			  $user
			&&!empty($this->users[$user])
		){
			unset($this->users[$user]);			
			if ($save){
				$this->save($this->users_data_folder.'users.php',$this->users);
			}
		}
	}

	# addUser
	###########################
  	public function addUser($user_data=null){
		if (
			  !$user_data
			||!is_array($user_data)
			||empty($user_data['login'])
			||empty($user_data['pass'])
		){
			return false;
		}

		if ($this->isLoginUsed($user_data['login'])){
			exit('login not available');
		}

		# Complete if needed		
		if (empty($user_data['email'])){
			$user_data['email']='';
		}
		# Add 
		$user_data['salt']=\tools::generateSalt(512);
		$user_data['pass']=\tools::hash($user_data['pass'],$user_data['salt']);

		$this->users[$user_data['login']]=$user_data;
		$this->save();
		
	}

	# changeUserData
	###########################
	public function changeUserData($login=null,$data_array=null,$save=true){
		if (
			  !isset($login)
			||!isset($this->users[$login])
			||empty($data_array)
		){
			return false;
		}	

		foreach ($data_array as $key => $value) {			
				if ($key=='login'){
					$userdata=$this->users[$login];
					$userdata['login']=$data;
					unset($this->users[$login]);
					$this->users[$value]=$userdata;			
				}elseif ($key=='pass'){
					$this->users[$login]['pass']=$this->hash($value, $this->users[$login]['salt']);	
				}else{
					$this->users[$login][$key]=$value;
				}
		}

		$this->secure->updateSession();

		if ($save){
			$this->save($this->users_data_folder.'users.php',$this->users);
		}

	}

	# sortUsers
	###########################
	public function sortUsers(){
		$first_data=reset($this->users);
		$first_login=$first_data['login'];
		$others=$this->users;
		unset($others[$first_login]);
		if (!empty($others)){
			ksort($others);
		}
		$this->users=array_merge(array($first_login=>$first_data),$others);
	}

	# isLoginUsed
	###########################
	## $login as string
	## Return : boolean
	public function isLoginUsed($login=null){
		if (empty($login)){return false;}
		return isset($this->users[$login]);
	}

	# count
	###########################
	## Return the amount of users
	public function count(){
		if (!empty($this->users)){
			return count($this->users);
		}else{return 0;}
	}

	# load
	###########################
	# load the config file
	## Return : array
	private function load(){
		$this->users=\tools::load($this->users_folder_path.'users.php');
	}

    # save
    # save the config file
    ###########################
    private function save(){
        $file=$this->users_folder_path.'users.php';
        return  \tools::save($file,$this->users);
    }
	
	public function setContext(&$context=null){					
		$this->context=&$context;
		$this->secure=\tools::getClassInstance($this->context,'secure');
	}	

	public function get($user){
		if (
			$user
			&&empty($this->users[$user])
		){
			return false;
		}
		if (!$user){
			return $this->users;
		}else{
			return $this->users[$user];
		}

	}
}
